%{
 /*! 
 * \file verilogscanner.l
 * \author Jonas Norling, Johan Lissing
 * \date 2005-11-27
 * \version 1.0 Added comments and this header.
 *
 * This is the Verilog parser.
 */

 /*
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

 /*!
  * To do:
  *
  * Arrays of module instances probably don't work.
  * Parse module instantiation port mapping.
  * Handle multiple instantiations of a module, comma separated.
  * Parse portlists in mod. decl. (the one in parenthesis), extract comments
  *   In verilog 2001, this may be the only list available (no input decls).
  *   This might require a small local symbol table with pointers to entries.
  * reg signed regname; // New in Verilog 2001. Are there more cases like this one?
  * I don't think that "reg[7:0] something;" works, we require whitespace
  *   after the net type.
  *
  */


#include "verilogscanner.h"
#include "entry.h"
#include "commentscan.h"
#include "message.h"
#include "doxygen.h"
#include "debug.h"
#include "iostream"
  using namespace std;


#define VERILOG_DEBUG
#ifdef VERILOG_DEBUG
#define debug(d) { \
		   if (debug_verilog) \
		      cout << yyFileName << ":" << yylineno \
			   << ": DEBUG " << d << endl; \
		 }
#else
#define debug(d)
#endif


#define YY_NEVER_INTERACTIVE 1

  /// Print debug data? Set by a command line argument.
  static bool debug_verilog = false;
  /// Points to the current parser (class) instance.
  static ParserInterface *g_thisParser;
  /// The entire input, received by caller.
  static const char *inputString;
  /// Read position in input buffer.
  static int inputPosition;

  static QCString yyFileName;
  static QFile inputFile;

  /// Current (or next) entry.
  static Entry*		current      = 0;
  /// Parent of current entry.
  static Entry*		parent       = 0;
  static Entry*		previous     = 0;

  /// Used when parsing module arguments
  static Argument*      argument     = 0;

  /// Current documentation block text.
  static QCString docBlock;

  // Counters for currnet block depth
  static int sequenceBlockDepth = 0; ///< Begin-end
  static int parallelBlockDepth = 0; ///< Fork-join
  static int caseBlockDepth = 0;     ///< Case-endcase

  /// Parenthesis matching counter
  static int parCounter = 0;



#undef	YY_INPUT
#define	YY_INPUT(buf, result, max_size) result=yyread(buf, max_size);

  /*!
   * Feed data to lex. Borrowed from pyscanner.l
   */
  static int yyread(char *buf,int max_size)
    {
      int c=0;
      while ( (c < max_size) && (inputString[inputPosition]) )
	{
	  *buf = inputString[inputPosition++] ;
	  //printf("%d (%c)\n",*buf,*buf);
	  c++;
	  buf++;
	}
      return c;
    }


  /*!
   * Create a new entry node. Call this when your'e done filling a node.
   */
  static void newEntry()
    {
      previous = current;
      current->parent->addSubEntry(current);

      current = new Entry;
      current->parent = parent;
      initGroupInfo(current);
    }

  /*!
   * Add a new argument to the current node's argument list.
   */
  static void newArgument()
    {
      current->argList->append(argument);
      argument = new Argument;
    }

  /*!
   * Reset current entry. You want to call this when
   * reversing a bad parsing decision.
   */
  static void resetEntry()
    {
      current->reset();
      current->parent = parent;
      initGroupInfo(current);
    }

  /*!
   * Call before newEntry() to descend in the hierarchy
   */
  static void descend()
    {
      parent = current;
    }

  /*!
   * Ascend in the (module or whatever) hierarchy.
   * Not expected to be called in conjunction with newEntry().
   */
  static void ascend()
    {
      parent = parent->parent;
      current->parent = parent;
    }

  /*!
   * Initialize information about a comment block.
   * \param brief True if this is a brief (one-line) comment.
   * \param pertainsTo The node that this comment is about, if the comment
   *        is given before a block, this is set to current. If the comment
   *        is given after a block (///<), this is set to previous.
   */
  static void startCommentBlock(bool brief, Entry *pertainsTo)
  {
    if (brief)
    {
      pertainsTo->briefFile = yyFileName;
      pertainsTo->briefLine = yylineno;
    }
    else
    {
      pertainsTo->docFile = yyFileName;
      pertainsTo->docLine = yylineno;
    }
  }

  /*!
   * Put a block of documentation in an entry.
   * \param doc		Documentation text
   * \param brief
   * \param pertainsTo	The Entry that this documentation pertains to (current or previous)
   */
  static void handleCommentBlock(const QCString &doc, bool brief, Entry *pertainsTo)
    {
      int position = 0;
      bool needsEntry;
      Protection protection = Public;

      while (parseCommentBlock(g_thisParser,
			       pertainsTo,
			       doc,
			       yyFileName,
			       brief ? pertainsTo->briefLine : pertainsTo->docLine,
			       brief,
			       false,
			       protection,
			       position,
			       needsEntry))
      {
	if (needsEntry)
        {
	  newEntry();
	}
      }
      if (needsEntry)
      {
	newEntry();
      }
    }

  /*!
   * Port declarations can be put in the module header and/or body. This
   * function is supposed to merge them. This is work in progress!
   * There will probably be a need for some kind of symbol table here.
   * The current module is found in parent.
   */
  static void mergePortDeclarations()
  {
	debug("Port list for module " << parent->name << ":");
	for (ArgumentListIterator ali(*parent->argList); ali.current(); ++ali){
		Argument* arg = ali.current();
		debug("  " << (arg->type ? arg->type : "notype") << " " <<
			(arg->name ? arg->name : "noname") << " ");
	}
  }

%}







%option noyywrap
%option stack
%option yylineno

VERILOGDIRECTIVE ("define"|"undef"|"include"|"ifdef"|"ifndef"|"else"|"endif"|"celldefine"|"endcelldefine"|"default_nettype"|"unconnected_drive"|"nounconnected_drive"|"resetall"|"timescale")

LETTER	[A-Za-z]
DIGIT	[0-9]

BBN             [ \t\n]+
BB		[ \t]+
B		[ \t]*

IDENTIFIER	({LETTER}|"_")({LETTER}|{DIGIT}|"_")*
VALUE           [+-]?[0-9a-fA-FxXzZ_]+
BASE            "'"[bBoOdDhH]
NUMBER          [{BASE}{NUMBER}]+

NUMTYPEKEYWORD time|integer|real
PORTKEYWORD input|output|inout
NETKEYWORD wire|tri|tri1|supply0|wand|triand|tri0|supply1|wor|trior|trireg|reg|parameter|localparam|genvar
GATEKEYWORD and|nand|or|nor|xor|xnor|buf|bufif0|bufif1|not|notif0|notif1|pulldown|pullup|nmos|rnmos|pmos|rpmos|cmos|rcmos|tran|rtran|tranif0|rtranif0|tranif1|rtranif1

BEHAVIORALKEYWORD initial|always
SIMPLESTATEMENTKEYWORD disable|assign|deassign|force|release|->
SIMPLEMODULEITEM assign|defparam|event|{GATEKEYWORD}
CASEKEYWORD case|casex|casez
LOOPKEYWORD repeat|while|for|wait

ASSIGNMENTOPERATOR "="|"<="




 /* Parser states (start conditions) */

%x ParserSkip

%s Search

%s UDPDec

%s ModuleDec
%s ModuleArgs
%s ArgList
%s ArgSizeRange
%s ModuleBody

%s Specify
%s Task
%s Function
%s Generate

%s SkipStatementToBody

%s Statement
%s Loop
%s If
%s DelayControl
%s EventControl
%s CaseStatement
%s SequenceBlock
%s ParallelBlock
%s Instantiation
%s SkipImplicitModParam

%s NumtypeDec
%s PortDec
%s NetDec
%s SizeRange
%s ArrayRange
%s ParamValue
%s Concatenation
%s Attribute
%s SkipCPP
%x SkipString

%x DocLine
%x AfterDocLine
%x DocRegion
%x AfterDocRegion
%x CommentLine
%x CommentRegion

%%











 /*
  * Global rules to handle comments anywhere in the code.
  */
"//" {
  yy_push_state(CommentLine);
}
"/*" {
  yy_push_state(CommentRegion);
}

 /*
  * Global rules for handling documentation
  * Doxygen will replace multi-line //-blocks with
  * C-style comment blocks before we get them.
  */
("///"|"//!")/[^"/"<] {  /* Catch all /// not followed by more slashes */
    docBlock.resize(0);
    startCommentBlock(true, current);
    yy_push_state(DocLine);
  }
("///<"|"//!<") {
    docBlock.resize(0);
    startCommentBlock(true, previous);
    yy_push_state(AfterDocLine);
  }
("/**"|"/*!") {
    docBlock.resize(0);
    startCommentBlock(false, current);
    yy_push_state(DocRegion);
  }
("/**<"|"/*!<") {
    docBlock.resize(0);
    startCommentBlock(false, previous);
    yy_push_state(AfterDocRegion);
  }


 /* Verilog 2001 (* attribute *) */
^"(*" {
  debug("Found an attribute");
  yy_push_state(Attribute);
}


 /*
  * Skip preprocessor context and defines
  * Doxygen's preprocessor replaces verilog compiler directives (`define, etc.)
  * with CPP style directives (#define, etc.) and kills off a lot of them.
  */
^{B}*"#"(define|" "{DIGIT}) {
  yy_push_state(SkipCPP);
}
^{B}*"\`"{VERILOGDIRECTIVE} { /* Skip preprocessor directives that might slip through */
  yy_push_state(SkipCPP);
}



 /*
  * Workaround for parser errors, if needed
  */
"//!"{B}"\\startskip" {
    yy_push_state(ParserSkip);
    debug("Starting to skip");
  }

 /*
  * Skip the contents of any string found
  */
"\"" {
  debug("Skipping string");
  yy_push_state(SkipString);
}







<ParserSkip>{
    "//!"{B}"\\stopskip" {
      yy_pop_state();
      debug("Done skipping");
    }
    .|"\n"
}

 /*
  * Start state, look for a module or primitive declaration.
  */
<Search>{
  "module"{BB} {
    BEGIN(ModuleDec);
  }
  "primitive"{BB} {
    BEGIN(UDPDec);
  }
  .
  \n
}

 /*
  * User-defined primitive, skip.
  */
<UDPDec>{ //Could be documented like a module, but skipped for now.
  "endprimitive"{B} {
    debug("Skipped a UDP");
    BEGIN(Search);
  }
  .
  \n
}

 /*
  * A module declaration has been found, hop through a few states to parse the
  * module declaration header.
  */
<ModuleDec>{
  {IDENTIFIER} {
    current->type = "module";
    current->argList->clear();
    current->section = Entry::MODULE_SEC;
    current->name = yytext;
    current->name = current->name.stripWhiteSpace();
    current->bodyLine  = yylineno;
    current->startLine = yylineno;
    current->fileName = yyFileName;        
    debug("Found module "+string(yytext));
    BEGIN(ModuleArgs);
  }
  {BB}
  \n
  . { warn(yyFileName, yylineno, "Warning: Unknown input in ModuleDec"); }
}

<ModuleArgs>{
  {BB}
  "(" {
    yy_push_state(ArgList);
  }
  ";" {
    descend(); newEntry();
    BEGIN(ModuleBody);
  }
  \n
  . { warn(yyFileName, yylineno, "Warning: Unknown input in ModuleArgs"); }
}

<ArgList>{
  {BB}
  input|output|inout {
    argument->type = yytext;
  }
  "[" {
    argument->type += " [";
    yy_push_state(ArgSizeRange);
  }
  {IDENTIFIER} {
    argument->name = yytext;
  }
  ")" {
    if (argument->name) newArgument();
    debug("End of arglist");
    yy_pop_state();
  }
  "," {
    newArgument();
  }
  \n
  . { warn(yyFileName, yylineno, "Warning: Unknown input in ArgList"); }
}

<ArgSizeRange>{
  \] {
    argument->type += "]";
    yy_pop_state();
  }
  . {
    argument->type += yytext;
  }
  \n { warn(yyFileName, yylineno, "Warning: Unexpected newline in port size range"); }
}

 /*
  * End of module header, parse body.
  */
<ModuleBody>{
  "endmodule"{B} {
    mergePortDeclarations();
    ascend();
    BEGIN(Search);
  }
  {PORTKEYWORD}{BB} { /* in, out, inout port */
    current->type = QCString(yytext).stripWhiteSpace();
    BEGIN(PortDec);
  }
  {NETKEYWORD}{BB} { /* Wires, regs, parameters */
    current->type = QCString(yytext).stripWhiteSpace();
    BEGIN(NetDec);
  }
  {NUMTYPEKEYWORD}{BB} { /* time, integer, real. */
    current->type = QCString(yytext).stripWhiteSpace();
    BEGIN(NumtypeDec);
  }
  {BEHAVIORALKEYWORD}{BBN} { /* initial or always block */
    debug("Starting a " << QCString(yytext).stripWhiteSpace() << " block");
    BEGIN(Statement);
  }
  {BEHAVIORALKEYWORD}{B}/@ { /* initial or always block */
    debug("Starting a " << QCString(yytext).stripWhiteSpace() << " block, seeing a @.");
    BEGIN(Statement);
  }
  {SIMPLEMODULEITEM}{B} { /* A single ;-terminated statement. Skip. */
    BEGIN(SkipStatementToBody);
  }
  "else"{B} {
    debug("Start of else statement");
    BEGIN(Statement);
  }
  "specify"{B} {
    debug("Start of specify block");
    BEGIN(Specify);
  }
  "task"{B} {
    debug("Start of task block");
    BEGIN(Task);
  }
  "function"{B} {
    debug("Start of function block");
    BEGIN(Function);
  }
  "generate"{B} {
    debug("Start of generate block");
    BEGIN(Generate);
  }
  {IDENTIFIER}{BB} { /* Module instantiation */
    debug("Instantiation of " << yytext);
    current->type = QCString(yytext).stripWhiteSpace();
    BEGIN(Instantiation);
  }
  ;
  .
  \n
}


 /*
  * Stuff that can be found in a module body but aren't interesting:
  */

<Specify>{
  "endspecify" {
    debug("End of specify block");
    BEGIN(ModuleBody);
  }
  .
  \n
}

<Task>{
  "endtask" {
    debug("End of task block");
    BEGIN(ModuleBody);
  }
  .
  \n
}

<Function>{
  "endfunction" {
    debug("End of function block");
    BEGIN(ModuleBody);
  }
  .
  \n
}

<Generate>{
  "endgenerate" {
    debug("End of generate block");
    BEGIN(ModuleBody);
  }
  .
  \n
}

<SkipStatementToBody>{
  ; { 
    debug("Skipped a statement");
    BEGIN(ModuleBody);
  }
  .
  \n
}

 /*
  * A behavioral statement has been found. Skip it.
  */
<Statement>{
  {SIMPLESTATEMENTKEYWORD}{BB} { //disable,assign,deassign,force,release
    BEGIN(SkipStatementToBody);
  }
  {CASEKEYWORD}({BB}|"(") { //case,casex,casez
    caseBlockDepth++;
    debug("Start of case statement " << caseBlockDepth);
    BEGIN(CaseStatement);
  }
  "if"{B} {
    debug("Start of if statement");
    BEGIN(If);
  }
  "begin"({BBN}|":") {
    sequenceBlockDepth++;
    debug("Start of sequence block " << sequenceBlockDepth);
    BEGIN(SequenceBlock);
  }
  "fork"{BBN} {
    parallelBlockDepth++;
    debug("Start of parallel block " << parallelBlockDepth);
    BEGIN(ParallelBlock);
  }
  LOOPKEYWORD{B} {
    debug("Start of loop statement");
    BEGIN(Loop);
  }
  "@"{B} {
    debug("Start of event control");
    BEGIN(EventControl);
  }
  "#"{B} {
    debug("Start of delay control");
    BEGIN(DelayControl);
  }
  {ASSIGNMENTOPERATOR}{B} {
    BEGIN(SkipStatementToBody);
  }
  "forever"
  "->"{B} {
    BEGIN(SkipStatementToBody);
  }
  .
  \n
}

 /*
  * Behavioral statements. Skip.
  */

<Loop>{
  "(" {
    parCounter++;
  }
  ")" {
    parCounter--;
    if (parCounter == 0)
    {
      debug("End of loop statement");
      BEGIN(Statement);
    }
  }
  .
  \n
}


<If>{
  "(" {
    parCounter++;
  }
  ")" {
    parCounter--;
    if (parCounter == 0)
    {
      debug("End of if statement");
      BEGIN(Statement);
    }
  }
  "if" { //"else if"
    debug("Start of if statement");
  }
  .
  \n
}


<DelayControl>{
  "(" {
    parCounter++;
  }
  ")" {
    parCounter--;
    if (parCounter == 0)
    {
      debug("End of delay control");
      BEGIN(Statement);
    }
  }
  {NUMBER}{B} {
    if (parCounter == 0)
    {
      debug("End of delay control");
      BEGIN(Statement);
    }
  }
  {IDENTIFIER}{B} {
    if (parCounter == 0)
    {
      debug("End of delay control");
      BEGIN(Statement);
    }
  }
  .
  \n
}

<EventControl>{
  "(" {
    parCounter++;
  }
  ")" {
    parCounter--;
    if (parCounter == 0)
    {
      debug("End of event control");
      BEGIN(Statement);
    }
  }
  {IDENTIFIER}{B} {
    if (parCounter == 0)
    {
      debug("End of event control");
      BEGIN(Statement);
    }
  }
  .
  \n
}    

<CaseStatement>{
  "endcase"{BBN} {
    debug("End of case statement" << caseBlockDepth);
    if (--caseBlockDepth == 0) BEGIN(ModuleBody); else yy_pop_state();
  }
  {CASEKEYWORD}{B} { //Nested case statement
    caseBlockDepth++;
    debug("Start of case statement" << caseBlockDepth);
    yy_push_state(CaseStatement);
  }
  ([^ \n\t;])*({BBN}|;)
  \n
}

<SequenceBlock>{ /* A begin-end block */
  "end"{BBN} {
    debug("End of sequence block " << sequenceBlockDepth);
    if (--sequenceBlockDepth == 0) BEGIN(ModuleBody); else yy_pop_state();
  }
  "begin"{BBN} { //Nested sequence block
    sequenceBlockDepth++;
    debug("Start of sequence block " << sequenceBlockDepth);
    yy_push_state(SequenceBlock);
  }
  "//".*"\n"
  "/*"(.|\n)*"*/"
  ([^ \n\t;:])*({BBN}|;|:)
  . { warn(yyFileName, yylineno, "Warning: Should this happen in SequenceBlock?"); }
}

<ParallelBlock>{ /* A fork-join block */
  "join"{BBN} {
    debug("End of parallel block " << parallelBlockDepth);
    if (--parallelBlockDepth == 0) BEGIN(ModuleBody); else yy_pop_state();
    yy_pop_state();
  }
  "fork"{BBN} { //Nested parallel block
    debug("Start of parallel block " << ++parallelBlockDepth);
    yy_push_state(ParallelBlock);
  }
  .
  \n
}


 /*
  * A module instantiation. Extract identifier and add to documentation.
  */
<Instantiation>{
  "#" {
    debug("Entering SkipImplicitModParam");
    yy_push_state(SkipImplicitModParam);
  }
  {IDENTIFIER} {
    debug("Found instance name " << yytext);
    current->section = Entry::VARIABLE_SEC;
    current->fileName  = yyFileName;
    current->bodyLine  = yylineno;
    current->startLine = yylineno;
    current->name = QCString(yytext).stripWhiteSpace();
    current->protection = Private;
    newEntry();
    BEGIN(SkipStatementToBody); // We should handle arrays of instances and parse arguments
  }
  {BB} { debug("Found some whitespace: --" << QCString(yytext) << "--"); }
  . {
    warn(yyFileName, yylineno, "Warning: Broken module instantiation, got " + QCString(yytext));
    resetEntry();
    BEGIN(ModuleBody);
  }
  \n {
    // warn(yyFileName, yylineno, "Warning: Newline in module instantiation, potential bug.");
  }
}

 /*
  * Module parameters given on #(stuff) form. Skip them.
  */
<SkipImplicitModParam>{
  "(" {
    parCounter++;
  }
  ")" {
    parCounter--;
    if (parCounter == 0) yy_pop_state();
  }
  ; { warn(yyFileName, yylineno, "Warning: Unexpected semicolon in implicit module params."); }
  \n
  .
}


 /*
  * Things that should be documented like variables:
  */

<NumtypeDec>{
  ; BEGIN(ModuleBody);
  {IDENTIFIER}{BB} {
    current->section = Entry::VARIABLE_SEC;
    current->fileName  = yyFileName;
    current->bodyLine  = yylineno;
    current->startLine = yylineno;
    current->name = QCString(yytext).stripWhiteSpace();
    current->protection = Private;
    newEntry();
  }
  , { //More declarations in same statement, copy type.
    current->type = previous->type;
  }
  \n
  .
}  

<PortDec>{
  {IDENTIFIER} {
    current->section = Entry::VARIABLE_SEC;
    current->fileName  = yyFileName;
    current->bodyLine  = yylineno;
    current->startLine = yylineno;
    current->name = QCString(yytext).stripWhiteSpace();
    current->protection = Public;
  }
  ; {
    newEntry();
    BEGIN(ModuleBody);
  }
  , { //More declarations in same statement, copy type.
    newEntry();
    current->type = previous->type;
  } 
  {BB}
  \n
  \[ {
    current->type += " [";
    yy_push_state(SizeRange);
  }
  . { warn(yyFileName, yylineno, "Warning: Unexpected garbage in port declaration"); }
}  

<NetDec>{
  "signed" { /* Register declared as signed */
    current->type += " ";
    current->type += yytext;
  }
  {IDENTIFIER} {
    current->section = Entry::VARIABLE_SEC;
    current->fileName  = yyFileName;
    current->bodyLine  = yylineno;
    current->startLine = yylineno;
    current->name = QCString(yytext).stripWhiteSpace();
    current->protection = Private;
  }
  ; { /* End of declarations */
    newEntry();
    BEGIN(ModuleBody);
  }
  , { // More declarations in same statement, copy type.
    newEntry();
    current->type = previous->type;
  } 
  {BB}
  \n
  \[ {
    if (!current->name) { /* If the current position is before the identifier, parse size */
      current->type += " [";
      yy_push_state(SizeRange);
    } else { /* Else parse array size */
      current->args += "[";
      yy_push_state(ArrayRange);
    }
  }
  = {
    yy_push_state(ParamValue);
  }
  . { warn(yyFileName, yylineno, "Warning: Unexpected garbage in net declaration"); }
}  

 /*
  * A bus-size range ([31:0]). Add to the data type.
  */
<SizeRange>{
  \] {
    current->type += "]";
    yy_pop_state();
  }
  . {
    current->type += yytext;
  }
  \n { warn(yyFileName, yylineno, "Warning: Unexpected newline in size range declaration"); }
}

 /*
  * An array range. Add to args.
  */
<ArrayRange>{
  \] {
    current->args += "]";
    yy_pop_state();
  }
  . {
    current->args += yytext;
  }
  \n { warn(yyFileName, yylineno, "Warning: Unexpected newline in array range declaration"); }
}

 /*
  * Value of a parameter or net. Put in the initializer.
  */
<ParamValue>{
  "{" {
    current->initializer += yytext;
    yy_push_state(Concatenation);
  }
  (,|;) { /* Let the enclosing state have the delimiter to decide what to do next */
    current->initializer = current->initializer.stripWhiteSpace();
    unput(*yytext);
    yy_pop_state();
  }
  (.|\n) {
    current->initializer += yytext;
  }
}

 /*
  * A { con, cat, en, ation }, found in a param value. Add to the initializer.
  */
<Concatenation>{
  "{" {
    current->initializer += yytext;
    yy_push_state(Concatenation);
  }
  "}" {
    current->initializer += yytext;
    yy_pop_state();
  }
  (.|\n) {
    current->initializer += yytext;
  }
}


 /*
  * Skip Verilog 2001 attributes.
  */
<Attribute>{
  "*)" {
    yy_pop_state();
  }
  .
  \n
}

 /*
  * Skip stray preprocessor directives.
  */
<SkipCPP>{
  \n {
    yy_pop_state();
  }
  .
}

<SkipString>{
  "\\\"" /* Escaped quotation mark */
  "\"" {
    yy_pop_state();
  }
  \n
  .
}


 /*
  * Start conditions for handling documentation:
  */

 /*
  * A single (brief) line of ducumentation.
  */
<DocLine>{
  .* {
    handleCommentBlock(yytext, true, current);
    debug("Docline: " << yytext);
    yy_pop_state();
  }
  \n {
    yy_pop_state();
  }
}

 /*
  * A single (brief) line of ducumentation, given after the member (<).
  */
<AfterDocLine>{
  .* {
    handleCommentBlock(yytext, true, previous);
    debug("After-Docline: " << yytext);
    yy_pop_state();
  }
  \n {
    yy_pop_state();
  }
}

 /*
  * A block of ducumentation.
  */
<DocRegion>{
  "*"*"*/" { /* A block may end in any number of stars */
    debug("DocRegion: " << docBlock);
    handleCommentBlock(docBlock, false, current);
    yy_pop_state();
  }
  ^{B}*"*"+/[^"/"] {  /* Skip stars at the beginning of a line */
  }
  . {
    docBlock += yytext;
  }
  \n  {
    docBlock += yytext;
  }
}

 /*
  * A block of ducumentation, given after the member (<).
  */
<AfterDocRegion>{
  "*"*"*/" { /* A block may end in any number of stars */
    debug("AfterDocRegion: " << docBlock);
    handleCommentBlock(docBlock, false, previous);
    yy_pop_state();
  }
  ^{B}*"*"+/[^"/"] {  /* Skip stars at the beginning of a line */
  }
  . {
    docBlock += yytext;
  }
  \n  { 
    docBlock += yytext;
  }
}


 /*
  * A // comment line, skip!
  */
<CommentLine>{
  .*
  \n yy_pop_state();
}

<CommentRegion>{
  "*/" {
    yy_pop_state();
  }
  .
  \n
}
%%








 /*!
  * Entry point for the parser. See verilogscanner.h.
  */
void VerilogLanguageScanner::parseInput(const char *fileName,
					const char *fileBuf,
					Entry *root)
{
  if (Debug::isFlagSet(Debug::Parser)) debug_verilog = true;

  g_thisParser = this;
  inputString = fileBuf;
  inputPosition = 0;
  sequenceBlockDepth = 0;
  caseBlockDepth = 0;
  parallelBlockDepth = 0;
  parCounter = 0;

  inputFile.setName(fileName);
  if (inputFile.open(IO_ReadOnly)) {
    yylineno = 1;
    yyFileName = fileName;
    msg("Parsing file %s...\n", yyFileName.data());

    argument = new Argument;

    current = new Entry;
    parent = root;
    current->parent = parent;

    /* Add an entry for this source file */
    current->name    = yyFileName;
    current->section = Entry::SOURCE_SEC;
    root->addSubEntry(current);
    current          = new Entry;
    current->reset();
    current->parent = parent;

    verilogscanYYrestart(verilogscanYYin);
    BEGIN(Search);
    verilogscanYYlex();

    if (YY_START != Search && YY_START != SkipCPP) {
      warn(yyFileName, yylineno, "Warning: Scanner in wrong state on EOF (%d), truncated file?", YY_START);
    }

    debug("Parsed " << yylineno << " lines.");

    if (sequenceBlockDepth != 0) warn(yyFileName, yylineno,
		 "Warning: sequenceBlockDepth = %d after parsing",
		 sequenceBlockDepth);
    if (caseBlockDepth != 0) warn(yyFileName, yylineno,
		 "Warning: caseBlockDepth = %d after parsing",
		 caseBlockDepth);
    if (parallelBlockDepth != 0) warn(yyFileName, yylineno,
		 "Warning: parallelBlockDepth = %d after parsing",
		 parallelBlockDepth);
    if (parCounter != 0) warn(yyFileName, yylineno,
		 "Warning: parCounter = %d after parsing",
		 parCounter);

    inputFile.close();
  }
}

bool VerilogLanguageScanner::needsPreprocessing(const QCString &)
{
  return true;
}


void VerilogLanguageScanner::parseCode(CodeOutputInterface &codeOutIntf,
				       const char *scopeName,
				       const QCString &input,
				       bool isExampleBlock,
				       const char *exampleName,
				       FileDef *fileDef,
				       int startLine,
				       int endLine,
				       bool inlineFragment,
				       MemberDef *memberDef)
{
}

void VerilogLanguageScanner::parsePrototype(const char *text)
{
}

void VerilogLanguageScanner::resetCodeParserState()
{
}
