/*! 
 * \file treeprinter.cpp
 * \author Thobias Bergkvist
 * \date 2005-11-15
 * \version 0.1 File created
 * \version 0.2 Added extended printing implementation.
 * \version 0.3 Added comments on variables ext and tab.
 * \version 0.3 Added exeption handling.
 * \version 0.3 Added overflow and underflow check of indent variable.
 * \version 0.3 Added trucated indent when tree depth > 10.
 * \version 0.3 Changed ? notation to if notation.
 *
 * A TreePrinter is used for printing a tree of Entry nodes.
 */

#include "./treeprinter.h"

/*!
 * The constructor.
 */
TreePrinter::TreePrinter(bool extended, int indent)
{
  ext = extended;

  if( ( indent < 0 ) || ( indent > 10 ) )
  {
    std::cerr << "TreePrinter: Invalide indent value, using default" << std::endl;
    tab = 3;
  }
  else
  {
    tab = indent;
  }
}

/*!
 * A public function used for printing a tree of Entry nodes.
 */
void TreePrinter::print(std::ostream& os, const Entry* node)
{
  try
  {
    print(os, node, 0);
  }
  catch(int err)
  {
    std::cerr << "TreePrinter: Unexpected exeption, code: " << err << std::endl;
  }
}

/*!
 * A private function used for printing out a tree of Entry nodes.
 * This function is called by the public print function.
 */
void TreePrinter::print(std::ostream& os, const Entry* node, int indent)
{
  // is there a node to print?
  if( !node ) return;

  // create the indent string
  std::string ind;
  if( indent == 0 )
  {
    ind = "";
  }
  else if( ( indent / tab ) > 10 )
  {
    ind = std::string( tab * 10 , ' ');
    os << ind << "<truncated indent>" << std::endl;
  }
  else
  {
    ind = std::string(indent, ' ');
  }

  // print entry information
  os << ind << "[ENTRY] {" << std::endl;
  if( node->fileName )
  {
    os << ind << " File: " << node->fileName << std::endl;
  }
  else
  {
    os << ind << " File: " << "-----" << std::endl;
  }
  if( node->name )
  {
    os << ind << " Name: " << node->name << std::endl;
  }
  else
  {
    os << ind << " Name: " << "-----" << std::endl;
  }
  if( node->type )
  {
    os << ind << " Type: " << node->type << std::endl;
  }
  else
  {
    os << ind << " Type: " << "-----" << std::endl;
  }
  if( node->doc )
  {
    os << ind << " Doc:  " << node->doc << std::endl;
  }
  else
  {
    os << ind << " Doc:  " << "-----" << std::endl;
  }

  // print extended entry information
  if( ext )
  {
    os << ind << " Section:     " << section( node->section ) << std::endl;

    if( node->args )
    {
      os << ind << " Args:        " << node->args << std::endl;
    }
    else
    {
      os << ind << " Args:        -----" << std::endl;
    }

    if( node->initializer )
    {
      os << ind << " Initializer: " << node->initializer << std::endl;
    }
    else
    {
      os << ind << " Initializer: -----" << std::endl;
    }

    os << ind << " StartLine:   " << node->startLine << std::endl;

    os << ind << " BodyLine:    " << node->bodyLine << std::endl;

    if( node->docFile )
    {
      os << ind << " DocFile:     " << node->docFile << std::endl;
    }
    else
    {
      os << ind << " DocFile:     -----" << std::endl;
    }

    os << ind << " DocLine:     " << node->docLine << std::endl;

    if( node->brief )
    {
      os << ind << " Brief:       " << node->brief << std::endl;
    }
    else
    {
      os << ind << " Brief:       -----" << std::endl;
    }

    if( node->briefFile )
    {
      os << ind << " BriefFile:   " << node->briefFile << std::endl;
    }
    else
    {
      os << ind << " BriefFile:   -----" << std::endl;
    }

    os << ind << " BriefLine:   " << node->briefLine << std::endl;

    if( node->includeFile )
    {
      os << ind << " IncludeFile: " << node->includeFile << std::endl;
    }
    else
    {
      os << ind << " IncludeFile: -----" << std::endl;
    }

    if( node->includeName )
    {
      os << ind << " IncludeName: " << node->includeName << std::endl;
    }
    else
    {
      os << ind << " IncludeName: -----" << std::endl;
    }

    switch( node->protection )
    {
    case Public:
      os << ind << " Protection:  public" << std::endl;
      break;
    case Protected:
      os << ind << " Protection:  protected" << std::endl;
      break;
    case Private:
      os << ind << " Protection:  private" << std::endl;
      break;
    case Package:
      os << ind << " Protection:  package" << std::endl;
      break;
    default:
      os << ind << " Protection:  -----" << std::endl;
    }
  }

  // print children
  if( node->sublist )
  {
    for(EntryListIterator eli( *node->sublist ); eli.current(); ++eli){
      print(os, eli.current(), indent + tab);
    }
  }

  // print end of entry
  os << ind << "}" << std::endl;
  return;
}

/*!
 * Translates an entry section to readable form.
 */
std::string TreePrinter::section(int section)
{

  std::string str;
  switch (section)
  {
  case Entry::CLASS_SEC:
    str = "CLASS_SEC";
    break;
  case Entry::STRUCT_SEC: 
    str = "STRUCT_SEC";
    break;
  case Entry::UNION_SEC: 
    str = "UNION_SEC";
    break;
  case Entry::EXCEPTION_SEC:
    str = "EXCEPTION_SEC";
    break;
  case Entry::NAMESPACE_SEC:
    str = "NAMESPACE_SEC";
    break;
  case Entry::INTERFACE_SEC:
    str = "INTERFACE_SEC";
    break;
  case Entry::PROTOCOL_SEC:
    str = "PROTOCOL_SEC";
    break;
  case Entry::CATEGORY_SEC:
    str = "CATEGORY_SEC";
    break;
  case Entry::MODULE_SEC:
    str = "MODULE_SEC";
    break;

  case Entry::CLASSDOC_SEC:
    str = "CLASSDOC_SEC";
    break;
  case Entry::STRUCTDOC_SEC:
    str = "STRUCTDOC_SEC";
    break;
  case Entry::UNIONDOC_SEC:
    str = "UNIONDOC_SEC";
    break;
  case Entry::EXCEPTIONDOC_SEC: 
    str = "EXCEPTIONDOC_SEC";
    break;
  case Entry::NAMESPACEDOC_SEC:
    str = "NAMESPACEDOC_SEC";
    break;
  case Entry::INTERFACEDOC_SEC:
    str = "INTERFACEDOC_SEC";
    break;
  case Entry::PROTOCOLDOC_SEC:
    str = "PROTOCOLDOC_SEC";
    break;
  case Entry::CATEGORYDOC_SEC:
    str = "CATEGORYDOC_SEC";
    break;
  case Entry::MODULEDOC_SEC:
    str = "MODULEDOC_SEC";
    break;

  case Entry::SOURCE_SEC:
    str = "SOURCE_SEC";
    break;
  case Entry::HEADER_SEC:
    str = "HEADER_SEC";
    break;

  case Entry::ENUMDOC_SEC:
    str = "ENUMDOC_SEC";
    break;
  case Entry::ENUM_SEC:
    str = "ENUM_SEC";
    break;
  case Entry::EMPTY_SEC:
    str = "EMPTY_SEC";
    break;
  case Entry::PAGEDOC_SEC:
    str = "PAGEDOC_SEC";
    break;
  case Entry::VARIABLE_SEC:
    str = "VARIABLE_SEC";
    break;
  case Entry::FUNCTION_SEC:
    str = "FUNCTION_SEC";
    break;
  case Entry::TYPEDEF_SEC:
    str = "TYPEDEF_SEC";
    break;
  case Entry::MEMBERDOC_SEC:
    str = "_MEMBERDOCSEC";
    break;
  case Entry::OVERLOADDOC_SEC:
    str = "OVERLOADDOC_SEC";
    break;
  case Entry::EXAMPLE_SEC:
    str = "EXAMPLE_SEC";
    break;
  case Entry::VARIABLEDOC_SEC:
    str = "VARIABLEDOC_SEC";
    break;
  case Entry::FILEDOC_SEC:
    str = "FILEDOC_SEC";
    break;
  case Entry::DEFINEDOC_SEC:
    str = "DEFINEDOC_SEC";
    break;
  case Entry::INCLUDE_SEC:
    str = "INCLUDE_SEC";
    break;
  case Entry::DEFINE_SEC:
    str = "DEFINE_SEC";
    break;
  case Entry::GROUPDOC_SEC:
    str = "GROUPDOC_SEC";
    break;
  case Entry::USINGDIR_SEC: 
    str = "USINGDIR_SEC";
    break;
  case Entry::MAINPAGEDOC_SEC:
    str = "MAINPAGEDOC_SEC";
    break;
  case Entry::MEMBERGRP_SEC:
    str = "MEMBERGRP_SEC";
    break;
  case Entry::USINGDECL_SEC:
    str = "USINGDECL_SEC";
    break;
  case Entry::PACKAGE_SEC:
    str = "PACKAGE_SEC";
    break;
  case Entry::PACKAGEDOC_SEC: 
    str = "PACKAGEDOC_SEC";
    break;
  case Entry::OBJCIMPL_SEC:
    str = "OBJCIMPL_SEC";
    break;
  case Entry::DIRDOC_SEC:
    str = "_DIRDOCSEC";
    break;

  default:
    {
      char s[16];
      sprintf(s, "0x%x", section);
      str = s;
    }
  }
  return str;
}

// EOF
